package net.apunch.blacksmith;

import net.apunch.blacksmith.util.Settings;
import net.citizensnpcs.api.npc.NPC;
import net.citizensnpcs.api.trait.Trait;
import net.citizensnpcs.api.util.DataKey;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.entity.LivingEntity;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.inventory.ItemStack;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.UUID;

public class BlacksmithTrait extends Trait {

    private final BlacksmithPlugin plugin;
    private final List<Material> reforgeAbleItems = new ArrayList<>();
    private final Map<UUID, Calendar> coolDowns = new HashMap<>();
    private ReforgeSession session;
    private final Settings config;

    public BlacksmithTrait() {
        super("blacksmith");
        plugin = (BlacksmithPlugin) Bukkit.getServer().getPluginManager().getPlugin("Blacksmith");
        this.config = BlacksmithPlugin.getInstance().getSettings();
    }

    public void addCoolDown(UUID playerUUID, Calendar waitUntil) {
        coolDowns.put(playerUUID, waitUntil);
    }

    public void unsetSession() {
        this.session = null;
    }

    @Override
    public void load(DataKey key) {
        for (DataKey sub : key.getRelative("reforgeable-items").getIntegerSubKeys()) {
            if (Material.getMaterial(sub.getString("").toUpperCase().replace('-', '_')) != null) {
                reforgeAbleItems.add(Material.getMaterial(sub.getString("").toUpperCase().replace('-', '_')));
            }
        }
        config.loadVariables(key);
    }

    @EventHandler
    public void onRightClick(net.citizensnpcs.api.event.NPCRightClickEvent event) {
        if (this.npc != event.getNPC()) {
            return;
        }

        Player player = event.getClicker();
        if ((config.getDisableCoolDown() & (coolDowns.get(player.getUniqueId()) != (null)))) {
            coolDowns.remove(player.getUniqueId());
        }
        if (!player.hasPermission("blacksmith.reforge")) {
            return;
        }

        if (coolDowns.get(player.getUniqueId()) != null) {
            if (!Calendar.getInstance().after(coolDowns.get(player.getUniqueId()))) {
                player.sendMessage(config.getCoolDownUnexpiredMessage());
                return;
            }
            coolDowns.remove(player.getUniqueId());
        }


        ItemStack hand = player.getInventory().getItemInMainHand();

        if (session != null) {
            //timeout
            if (System.currentTimeMillis() > _sessionStart + 10 * 1000 || this.npc.getEntity().getLocation().distance(session.getPlayer().getLocation()) > 20) {
                session = null;
            }
        }


        if (session != null) {
            if (!session.isInSession(player)) {

                player.sendMessage(config.getBusyWithPlayerMessage());
                return;

            }

            if (session.isRunning()) {
                player.sendMessage(config.getBusyReforgingMessage());
                return;
            }
            if (session.handleClick()) {
                session = null;
            } else {
                reforge(npc, player);
            }
        } else {
            if ((!plugin.isTool(hand) && !plugin.isArmor(hand))
                    || (!reforgeAbleItems.isEmpty() && !reforgeAbleItems.contains(hand.getType()))) {
                player.sendMessage(config.getInvalidItemMessage());
                return;
            }

            String cost = plugin.formatCost(player);

            _sessionStart = System.currentTimeMillis();
            session = new ReforgeSession(this, player, npc, config);
            player.sendMessage(config.getCostMessage().replace("<price>", cost).replace("<item>",
                    hand.getType().name().toLowerCase().replace('_', ' ')));

        }
    }

    private long _sessionStart = System.currentTimeMillis();

    @Override
    public void save(DataKey key) {
        for (int i = 0; i < reforgeAbleItems.size(); i++) {
            key.getRelative("reforgeable-items").setString(String.valueOf(i),
                    reforgeAbleItems.get(i).name().toLowerCase().replace('_', '-'));
        }

        config.saveVariables(key);
    }

    private void reforge(NPC npc, Player player) {
        player.sendMessage(config.getStartReforgeMessage());

        //plugin.deposit(npc, player); // CitiTrader dependency outdated and broken

        plugin.withdraw(player);
        session.beginReforge();
        if (npc.getEntity() instanceof Player) {
            ((Player) npc.getEntity()).getInventory().setItemInMainHand(player.getInventory().getItemInMainHand());
        } else {
            Objects.requireNonNull(((LivingEntity) npc.getEntity()).getEquipment()).setItemInMainHand(player.getInventory().getItemInMainHand());
        }
        player.getInventory().setItemInMainHand(null);
    }

}
